@@ -204,9 +204,11 @@ class MutwoPitchToHEJIAbjadPitchConverter(MutwoPitchToAbjadPitchConverter):
                     else self._utonality_indicator
                 )
                 heji_accidental_name = self._prime_to_heji_accidental_name[prime]
-                exponent = self._exponent_to_exponent_indicator(abs(exponent) - 1)
+                exponent_indicator = self._exponent_to_exponent_indicator(
+                    abs(exponent) - 1
+                )
                 accidental_parts.append(
-                    "{}{}{}".format(tonality, heji_accidental_name, exponent)
+                    "{}{}{}".format(tonality, heji_accidental_name, exponent_indicator)
                 )
 
         accidental = _HEJIAccidental("".join(accidental_parts))
@@ -223,7 +225,7 @@ class MutwoPitchToHEJIAbjadPitchConverter(MutwoPitchToAbjadPitchConverter):
         # the reference_pitch, but which are slightly deeper
         elif (
             closest_pythagorean_pitch_index == self._reference_index
-            and pitch_to_convert.normalize(mutate=False).cents > 600
+            and pitch_to_convert.normalize(mutate=False).cents > 600  # type: ignore
         ):
             octave += 1
 
@@ -370,7 +372,9 @@ class ComplexTempoEnvelopeToAbjadAttachmentTempoConverter(
 
     @staticmethod
     def _shall_stop_dynamic_change_indication(
-        tempo_attachments: typing.Tuple[abjad_attachments.Tempo, ...]
+        tempo_attachments: typing.Tuple[
+            typing.Tuple[constants.Real, abjad_attachments.Tempo], ...
+        ]
     ) -> bool:
         stop_dynamic_change_indicaton = False
         for _, previous_tempo_attachment in reversed(tempo_attachments):
@@ -431,7 +435,9 @@ class ComplexTempoEnvelopeToAbjadAttachmentTempoConverter(
         nth_tempo_point: int,
         tempo_point: parameters.tempos.TempoPoint,
         tempo_points: typing.Tuple[parameters.tempos.TempoPoint, ...],
-        tempo_attachments: typing.Tuple[abjad_attachments.Tempo, ...],
+        tempo_attachments: typing.Tuple[
+            typing.Tuple[constants.Real, abjad_attachments.Tempo], ...
+        ],
     ) -> abjad_attachments.Tempo:
         try:
             next_tempo_point: typing.Optional[
@@ -486,7 +492,9 @@ class ComplexTempoEnvelopeToAbjadAttachmentTempoConverter(
             tempo_envelope_to_convert.levels
         )
 
-        tempo_attachments = []
+        tempo_attachments: typing.List[
+            typing.Tuple[constants.Real, abjad_attachments.Tempo]
+        ] = []
         for nth_tempo_point, absolute_time, duration, tempo_point in zip(
             range(len(tempo_points)),
             tools.accumulate_from_zero(tempo_envelope_to_convert.durations),
@@ -566,11 +574,10 @@ class SequentialEventToQuantizedAbjadContainerConverter(converters_abc.Converter
             )
             raise ValueError(message)
 
-        # nauert will raise an error if there is only one time signature
-        elif n_time_signatures == 1:
-            time_signatures += time_signatures
-
         time_signatures = tuple(time_signatures)
+        # nauert will raise an error if there is only one time signature
+        if n_time_signatures == 1:
+            time_signatures += time_signatures
 
         if tempo_envelope is None:
             tempo_envelope = expenvelope.Envelope.from_points(
@@ -1036,11 +1043,11 @@ class SequentialEventToAbjadVoiceConverter(converters_abc.Converter):
         collection can be extracted), mutwo will build a notation indicator collection
         from :const:`~mutwo.events.music_constants.DEFAULT_NOTATION_INDICATORS_COLLECTION_CLASS`
     :type simple_event_to_notation_indicators: typing.Callable[[events.basic.SimpleEvent], parameters.notation_indicators.NotationIndicatorCollection,], optional
-    :param does_extracted_data_indicate_rest: Function to detect from the extracted
-        data if the inspected :class:`mutwo.events.basic.SimpleEvent` is a Rest. By
+    :param is_simple_event_rest: Function to detect if the
+        the inspected :class:`mutwo.events.basic.SimpleEvent` is a Rest. By
         default Mutwo simply checks if 'pitch_or_pitches' contain any objects. If not,
         the Event will be interpreted as a rest.
-    :type does_extracted_data_indicate_rest: typing.Callable[[typing.List[parameters.abc.Pitch], parameters.abc.Volume, parameters.playing_indicators.PlayingIndicatorCollection, parameters.notation_indicators.NotationIndicatorCollection,],bool,], optional
+    :type is_simple_event_rest: typing.Callable[[events.basic.SimpleEvent], bool], optional
     :param mutwo_pitch_to_abjad_pitch_converter: Class which defines how to convert
         :class:`mutwo.parameters.abc.Pitch` objects to :class:`abjad.Pitch` objects.
         See :class:`MutwoPitchToAbjadPitchConverter` for more information.
@@ -1077,18 +1084,7 @@ class SequentialEventToAbjadVoiceConverter(converters_abc.Converter):
             [events.basic.SimpleEvent],
             parameters.notation_indicators.NotationIndicatorCollection,
         ] = lambda simple_event: simple_event.notation_indicators,  # type: ignore
-        does_extracted_data_indicate_rest: typing.Callable[
-            [
-                typing.List[parameters.abc.Pitch],
-                parameters.abc.Volume,
-                parameters.playing_indicators.PlayingIndicatorCollection,
-                parameters.notation_indicators.NotationIndicatorCollection,
-            ],
-            bool,
-        ] = lambda pitches, volume, playing_indicators, notation_indicators: len(
-            pitches
-        )
-        == 0,
+        is_simple_event_rest: typing.Callable[[events.basic.SimpleEvent], bool] = None,
         mutwo_pitch_to_abjad_pitch_converter: MutwoPitchToAbjadPitchConverter = MutwoPitchToAbjadPitchConverter(),
         mutwo_volume_to_abjad_attachment_dynamic_converter: MutwoVolumeToAbjadAttachmentDynamicConverter = MutwoVolumeToAbjadAttachmentDynamicConverter(),
         tempo_envelope_to_abjad_attachment_tempo_converter: TempoEnvelopeToAbjadAttachmentTempoConverter = ComplexTempoEnvelopeToAbjadAttachmentTempoConverter(),
@@ -1101,6 +1097,16 @@ class SequentialEventToAbjadVoiceConverter(converters_abc.Converter):
         else:
             abjad_attachment_classes = tuple(abjad_attachment_classes)
 
+        if is_simple_event_rest is None:
+
+            def is_simple_event_rest(simple_event: events.basic.SimpleEvent) -> bool:
+                try:
+                    pitch_or_pitches = simple_event_to_pitches(simple_event)
+                except AttributeError:
+                    pitch_or_pitches = []
+
+                return not bool(pitch_or_pitches)
+
         self._abjad_attachment_classes = abjad_attachment_classes
 
         self._available_abjad_attachments = tuple(
@@ -1115,7 +1121,7 @@ class SequentialEventToAbjadVoiceConverter(converters_abc.Converter):
         self._simple_event_to_volume = simple_event_to_volume
         self._simple_event_to_playing_indicators = simple_event_to_playing_indicators
         self._simple_event_to_notation_indicators = simple_event_to_notation_indicators
-        self._does_extracted_data_indicate_rest = does_extracted_data_indicate_rest
+        self._is_simple_event_rest = is_simple_event_rest
         self._mutwo_pitch_to_abjad_pitch_converter = (
             mutwo_pitch_to_abjad_pitch_converter
         )
@@ -1531,14 +1537,23 @@ class SequentialEventToAbjadVoiceConverter(converters_abc.Converter):
             }
         }
         """
-        # first, extract data from simple events
+
+        # tie rests before processing the event!
+        sequential_event_to_convert = sequential_event_to_convert.tie_by(
+            lambda event0, event1: self._is_simple_event_rest(event0)
+            and self._is_simple_event_rest(event1),
+            event_type_to_examine=events.basic.SimpleEvent,
+            mutate=False,
+        )
+
+        # first, extract data from simple events and find rests
         extracted_data_per_simple_event = tuple(
             self._extract_data_from_simple_event(simple_event)
             for simple_event in sequential_event_to_convert
         )
         is_simple_event_rest_per_simple_event = tuple(
-            self._does_extracted_data_indicate_rest(*extracted_data)
-            for extracted_data in extracted_data_per_simple_event
+            self._is_simple_event_rest(simple_event)
+            for simple_event in sequential_event_to_convert
         )
 
         # second, quantize the sequential event
